/*
How to use the Razor as an I2C slave for providing orientation data to a master device. 
Written by user randyrover on the SparkFun forums and provided as open-source software. (Thanks!)
https://forum.sparkfun.com/viewtopic.php?f=83&t=52576
*/

#include <SparkFunMPU9250-DMP.h>

#include <Wire.h>

#include "wiring_private.h" // pinPeripheral() function

#include <HexCommands.h>//TransferDataBag definition

#define I2C_PIN_SDA 11

#define I2C_PIN_SCL 13

#define MSG_SIZE 6

#define I2C_RAZOR_SLAVE_ADDRESS  0x09

#define LOG_DELAY 2000


#define DMP_SAMPLE_RATE    100 // Logging/DMP sample rate(4-200 Hz)

#define IMU_COMPASS_SAMPLE_RATE  100 // Compass sample rate (4-100 Hz)

#define ENABLE_GYRO_CALIBRATION  true

#define IMU_GYRO_FSR       2000 // Gyro full-scale range (250, 500, 1000, or 2000)

#define IMU_ACCEL_FSR      2 // Accel full-scale range (2, 4, 8, or 16)

#define IMU_AG_LPF         5 // Accel/Gyro LPF corner frequency (5, 10, 20, 42, 98, or 188 Hz)

#define IMU_AG_SAMPLE_RATE  100 // Accel/gyro sample rate Must be between 4Hz and 1kHz


// Danger - don't change unless using a different platform

#define MPU9250_INT_PIN  4

#define MPU9250_INT_ACTIVE  LOW


#define DEBUG true

#if DEBUG

#define SerialPort SerialUSB

long log_timestamp;

#endif

//used to transfer data

union transferData

{

        uint16_t int_data[3];

        uint8_t byte_data[sizeof(int_data)];

} TransferDataBag;


//How to establish additional i2c connection with Arduino

//on Razor, use: pin 13 as SCL, 11 as SDA

//On Arduino side you need: Wire.begin(); to start wire

//and Wire.requestFrom(I2C_RAZOR_SLAVE_ADDRESS, MSG_SIZE) to request data with MSG_SIZE size

TwoWire razorWire(&sercom1, I2C_PIN_SDA, I2C_PIN_SCL);

//add SERCOM handler on SERCOM1

void SERCOM1_Handler(void)

{

  razorWire.onService();

}

MPU9250_DMP imu;


void setup()

{

  //init RazorWire instance as slave at 0x09 address

  razorWire.begin(I2C_RAZOR_SLAVE_ADDRESS);

  // Assign pins I2C_PIN_SDA & I2C_PIN_SCL to SERCOM functionality

  pinPeripheral(I2C_PIN_SDA, PIO_SERCOM);

  pinPeripheral(I2C_PIN_SCL, PIO_SERCOM);

  //add i2c request handler to process requests from i2c master

  razorWire.onRequest(ProcessWireRequest);

#if DEBUG

  SerialPort.begin(115200);

  Log("Setup: Start(after i2c initialization)");

#endif

  // Set up MPU-9250 interrupt input (active-low)

  pinMode(MPU9250_INT_PIN, INPUT_PULLUP);

  // Initialize the MPU-9250. Should return true on success:

  if ( !initIMU() )

  {

    while (1)  // Loop forever if we fail to connect

    {

      Log("Error connecting to MPU-9250");

    }

  }

}


void FillDataForTransfer()

{

  //we're preparing float data as int (when it's need we just divide by 100 to get float)

  TransferDataBag.int_data[0] = imu.yaw * 100;

  TransferDataBag.int_data[1] = imu.pitch * 100;

  TransferDataBag.int_data[2] = imu.roll * 100;

}


void WriteData()

{

  for (byte i = 0; i < MSG_SIZE; i++)

  {

    razorWire.write(TransferDataBag.byte_data[i]);

  }

}

//i2c requests handler

void ProcessWireRequest()

{

  FillDataForTransfer();

  WriteData();

}


bool initIMU(void)

{

  // imu.begin() should return 0 on success. Will initialize

  // I2C bus, and reset MPU-9250 to defaults.

  if (imu.begin() != INV_SUCCESS)

    return false;

  // Set up MPU-9250 interrupt:

  imu.enableInterrupt(); // Enable interrupt output

  imu.setIntLevel(1);    // Set interrupt to active-low

  imu.setIntLatched(1);  // Latch interrupt output

  // Configure sensors:

  // Set gyro full-scale range: options are 250, 500, 1000, or 2000:

  imu.setGyroFSR(IMU_GYRO_FSR);

  // Set accel full-scale range: options are 2, 4, 8, or 16 g

  imu.setAccelFSR(IMU_ACCEL_FSR);

  // Set gyro/accel LPF: options are5, 10, 20, 42, 98, 188 Hz

  imu.setLPF(IMU_AG_LPF);

  // Set gyro/accel sample rate: must be between 4-1000Hz

  // (note: this value will be overridden by the DMP sample rate)

  imu.setSampleRate(IMU_AG_SAMPLE_RATE);

  // Set compass sample rate: between 4-100Hz

  imu.setCompassSampleRate(IMU_COMPASS_SAMPLE_RATE);


  // Configure digital motion processor. Use the FIFO to get

  // data from the DMP.

  unsigned short dmpFeatureMask = 0;

  if (ENABLE_GYRO_CALIBRATION)

  {

    // Gyro calibration re-calibrates the gyro after a set amount

    // of no motion detected

    dmpFeatureMask |= DMP_FEATURE_GYRO_CAL;

    dmpFeatureMask |= DMP_FEATURE_SEND_CAL_GYRO;

  }

  else

  {

    // Otherwise add raw gyro readings to the DMP

    dmpFeatureMask |= DMP_FEATURE_SEND_RAW_GYRO;

  }

  // Add accel and quaternion's to the DMP

  dmpFeatureMask |= DMP_FEATURE_SEND_RAW_ACCEL;

  dmpFeatureMask |= DMP_FEATURE_6X_LP_QUAT;


  // Initialize the DMP, and set the FIFO's update rate:

  imu.dmpBegin(dmpFeatureMask, DMP_SAMPLE_RATE);

  return true; // Return success

}

//logging procedure

void Log(String str)

{

#if DEBUG

  if (SerialUSB)

    SerialPort.println(str); // Print log line to serial port

#endif

}

//logging procedure

void Log()

{

#if DEBUG

  if (millis() - log_timestamp > LOG_DELAY)

  {

    if (SerialUSB)

      Log("Yaw:" + String(imu.yaw) + ", Pitch:" + String(imu.pitch) + ", Roll:" + String(imu.roll)); // Print log line to serial port

    log_timestamp = millis();

    //blinkLED();

  }

#endif

}

void loop()

{

  // Then check IMU for new data, and log it

  if ( !imu.fifoAvailable() ) // If no new data is available

    return;                   // return to the top of the loop

  // Read from the digital motion processor's FIFO

  if ( imu.dmpUpdateFifo() != INV_SUCCESS )

    return; // If that fails (uh, oh), return to top

  // If enabled, read from the compass.

  if ( imu.updateCompass() != INV_SUCCESS)

    return; // If compass read fails (uh, oh) return to top

  imu.computeEulerAngles();

#if DEBUG

  Log();

#endif

}
